/**	CPPImageFilterJNI.cpp

	Ivo Dinov, Ph.D.
	http://www.stat.ucla.edu/~dinov

	Java C++ interface using JNI

	Sample code illustrating:
		- Java/C++ interface
		- parameter transfer
		- image filtering
		- image visualization

	Date: Jan. 12, 2004

	Publically available under GNU License

 */

#include <jni.h>
#include "JavaDriverJNI.h"
#include <stdio.h>

JNIEXPORT jbyteArray JNICALL 
Java_JavaDriverJNI_filterImage(JNIEnv *env, jobject obj, jbyteArray img )
{	char codebase[] = "CPPImageFilterJNI";
	jint kernel_size = 10;	// Kernel size for smoothing filter, in voxels.

    	fprintf(stderr, "\n\tCall from C++ (%s.cpp) ::: JavaDriverJNI 2!\n", codebase);

	jsize len = env->GetArrayLength(img);
 	/* This is C++ synthax, for C synthax use: len = (*env)->GetArrayLength(env,arr); */

	jint i, j, counter=0, sum=0; 
	jbyte *img_filtered = new jbyte[len];

	jbyte *buf = env->GetByteArrayElements(img, NULL);
	if (buf == NULL) return NULL; 



	/***************************************** MAIN IMAGE FILTER BEGIN *********************/
		// Filter the input image (img) - Example - smoothing with kernel of size kernel_size
	for (i = 0; i < len; i++)  
	{	counter = 0;
		sum=0;

		for (j=i-kernel_size; j < i+kernel_size; j++)
		{     if (j>=0 && j<len)
			{	sum += buf[j];
				counter++;
			 }
		 }
		if (counter >0)	img_filtered[i] = (jbyte) (2*sum/counter);
		else img_filtered[i] = 0;
	 }
	/***************************************** MAIN IMAGE FILTER END *********************/



		// Free memory
	//env->ReleaseByteArrayElements(img, buf, 0); 

		//Return filtered byte image (img_filtered)
	jbyteArray img_filteredJBA = env->NewByteArray(len);
	env->SetByteArrayRegion( img_filteredJBA, 0, len, img_filtered);
     	return img_filteredJBA; 
}
