/**	CPPStringFilterJNI.cpp

	Ivo Dinov, Ph.D.
	http://www.stat.ucla.edu/~dinov

	Java C++ interface using JNI

	Sample code illustrating:
		- Java/C++ interface
		- parameter transfer
		- string manipulations
		- array manipulation

	Date: Dec. 18, 2003

	Publically available under GNU License

 */

#include <jni.h>
#include "JavaDriverJNI.h"
#include <stdio.h>

#define StringMaxLength 1000

typedef char* CharStar;

JNIEXPORT jobjectArray JNICALL 
Java_JavaDriverJNI_processString(JNIEnv *env, jobject obj, jobjectArray objectArray)
{
	char codebase[] = "CPPStringFilterJNI";

	jobjectArray result;
    	jsize len1 = env->GetArrayLength(objectArray);
 	/* This is C++ synthax, for C synthax use: len = (*env)->GetArrayLength(env,arr); */
	CharStar* argv;
	int len = (int)(len1);

	fprintf(stderr, "\n\tCall from C++ (%s.cpp), size = %d ::: JavaDriverJNI 4!\n", codebase, len);

   	int i, k, local_len;

	argv = new CharStar[len];
	for (i = 0; i < len; i++)
	   argv[i] = new char[StringMaxLength];
	 
	for (i = 0; i < len; i++)	// For each input argument
	{    	jstring oneArgument = (jstring)(env->GetObjectArrayElement(objectArray, i));
		//fprintf(stderr, "\n\tCall from C++ (%s.cpp) ::: args-IN:: %s \n",codebase, oneArgument);

		int len2 = (int)(env->GetStringLength(oneArgument)); 
		CharStar str;
		env->GetStringUTFRegion(oneArgument , 0, len2, str);

		fprintf(stderr, "\n\tCall from C++ (%s.cpp) 5 str = %s", codebase, str);

		k=0;
		while (str[k] != '\0')
			k++;
		local_len=k;
		for (k = local_len-1; k >= 0; k--)
			argv[len-1-i][local_len-1-k] = str[k];
		argv[len-1-i][local_len] = '\0';
		env->ReleaseStringUTFChars(oneArgument, str);

		//fprintf(stderr, "\n\tCall from C++ (%s.cpp) 6 str = %s", codebase,argv[len-1-i]);
	 }
	
	result = env->NewObjectArray(len, env->FindClass("java/lang/String"), env->NewStringUTF(""));
	for(i = 0; i < len; i++) 
		env->SetObjectArrayElement( result, i, env->NewStringUTF(argv[i]));

	/**** Check to see that the newly constructed Java Object array (of strings) is well-defined ****
	jstring oneArgument = (jstring)(env->GetObjectArrayElement(result, 0));
	int len2 = (int)(env->GetStringLength(oneArgument)); 
	CharStar str;
	env->GetStringUTFRegion(oneArgument , 0, len2, str);
	fprintf(stderr, "\n\tCall from C++ (%s.cpp)7:::args-IN:: %s \n", codebase, str);
	********/

		// Release memory back to system
	for (i = 0; i < len; i++)
	   delete [] argv[i];
	delete [] argv;

	return result; 
}
