/**	JavaDriverJNI.java

	Ivo Dinov, Ph.D.
	http://www.stat.ucla.edu/~dinov

	Java C++ interface using JNI

	Sample code illustrating:
		- Java/C++ interface
		- parameter transfer
		- image filtering
		- image visualization

	Notes: Do not use "_" in the file-names, conflicts the JNI synthax; 
		 C and C++ JNI calls are slightly different see code for details
		 JNI Docs are at: http://java.sun.com/docs/books/jni/html/jniTOC.html

	Date: Jan. 12, 2004
	Publically available under GNU License
 */


class JavaDriverJNI {
    		// Prototypes of the Two Native Method Calls to external C++
	public native byte[] filterImage(byte[] img);
	private native String[] processString(String[] javaArgs); 

	private static String codebase = "JavaDriverJNI";

    	static { 
		// 1. Load the NAtive Array Method Library       
		try { System.loadLibrary("CPPImageFilterJNI"); }
		catch (java.lang.UnsatisfiedLinkError e)
		{ 	System.out.println ("Linking problem in "+codebase+".java - CPPImageFilterJNI :: "+e);  }

		// 2. Load the Native String Method Library
		try { System.loadLibrary("CPPStringFilterJNI"); }
		catch (java.lang.UnsatisfiedLinkError e)
		{ 	System.out.println ("Linking problem in "+codebase+".java - CPPStringFilterJNI :: "+e);  }
    	}
    
	private static byte img[], img_filtered[];

    	public static void main(String[] args) {
		// Synthax:  %> java JavaDriverJNI volume3D.img filterIndex [other args]
		// Example:  %> java JavaDriverJNI C:/Ivo.dir/Research/talks/STAT_UCLA_Fall01/Ivo_StatUCLA_01/Ivo_BioStat_UCLA1999/registr_grid_warped.gif

		/****** For ANALYZE image format - needs work (parser) ***************
			// 1. Call volume header!!!!
      		// 2. Processing of the image from within Java code
			System.out.println("\n\t Call from Java code ("+codebase+".java) "+
				"::: "+codebase+" 1\n");
			img_filtered = localImageFilter(img); 
				System.out.println("\t\tLocal Image Filter ("+codebase+".java) = "); 
			//  3.  Processing/Filtering of image from within C++ methods
			JavaDriverJNI newJavaDriver = new JavaDriverJNI();
			img_filtered = newJavaDriver.filterImage(img);
			System.out.println("\t\tCPP_Filtered Image ("+codebase+".java)  = "); 
			//  4. Display input and filtered images
			displayImages(img, img_filtered);
		**********************************************/

		// 1. Process the array/image data
		JavaShowImages JSI = new JavaShowImages(args[0]);
		JavaDriverJNI newJavaDriver = new JavaDriverJNI();
		img_filtered = newJavaDriver.filterImage(JSI.getArrayFromImage(JSI.srcIm));
		JSI.destIm = JSI.makeImageFromArray(img_filtered);
		System.out.println("\t\tCPP_Filtered Image ("+codebase+".java)  = "); 
		JSI.showBothImages(JSI.srcIm,JSI.destIm, "Input **************** Output (Filtered Image by CPP Code)");

		// 2. Process the String (input arguments, as an example)
		//    from within C++ methods
		    System.out.println("Before Call to processString");
		    String [] stringProcessed = newJavaDriver.processString(args);
		    System.out.println("After Call to processString");
		    String rev_string_args;

		    rev_string_args = "";
		    for (int i = 0; i< stringProcessed.length; i++)
			rev_string_args += stringProcessed[i];
		    System.out.println("\n\tCPP_Computed_ReversedString ("+codebase+".java)="+ rev_string_args); 

		return;
    }

	static byte[] localImageFilter(byte[] _img)
	{  	// For local testing of the image-filter from within this Java driver  	
		return _img;
	}

	static void displayImages(byte[] _img, byte[] _image_filtered)
	{  		// Display method shows starting image and a homotopy of the starting
			// image to the filtered image
		JavaShowImages JSI = new JavaShowImages();
		return;
	}

}	// END
